(* 
 A simple gray-image processing library

 A simpler and more elegant version of 
    http://okmij.org/ftp/Graphics.html#cpp.improc

 An image is a pixel map, a matrix of pixels arranged row-by-row.
 Each pixel specifies the gray level at a particular point. A row or
 a column index always starts with 0.

 For simplicity we assume the image depth to be 8 (that is, 
 the pixel range [0..255])
*)

type pixel = int
type img

val ncols : img -> int  (* Image width in pixels *)
val nrows : img -> int  (* Image height in pixels *)

(* A view: All pixels, in row major order *)
val pixels : img -> 
  (int,Bigarray.int8_unsigned_elt,Bigarray.c_layout) Bigarray.Array1.t

(* Reading and writing the image in the binary portable grayscale format
   (binary PGM)
*)

val read_pgm  : string -> img
val write_pgm : string -> img -> unit

(* Display the image with a particular command. The command
   should take the name of the image file, in PGM, as the argument.
*)

val display : ?cmd:string -> img -> unit

