(* Liftable types
   The types of present-stage values that are OK to use at a future
   stage (values safe to CSP, both in byte- and native-code modes)

   Although polymorphic lift is possible in MetaOCaml and works well
   in byte-code, it is not at all helpful in the native mode.
*)

module type LIFT = sig
  type t
  val lift: t -> t code
end

(* Although the lift methods below all look the same, it is the
   type that matters.
   The type will make MetaOCaml generate the right code for lift.
*)

module Lift_int = struct
  type t = int
  let lift (x:int) = .<x>.
end

module Lift_float = struct
  type t = float
  let lift (x:float) = .<x>.
end

module Lift_string = struct
  type t = string
  let lift (x:string) = .<x>.
end


(* More can, and should be added *)

module Lift_option(L:LIFT) = struct
  type t = L.t option
  let lift = function 
    | None   -> .<None>.
    | Some x -> .<Some .~(L.lift x)>.
end

module Lift_list(L:LIFT) = struct
  type t = L.t list
  let lift l = 
    Print_code.genlet @@
    List.fold_right (fun x acc -> .<.~(L.lift x) :: .~acc>.) l .<[]>.
end


